@@ -36,7 +36,7 @@ consistent-looking plots.
 
 """
 
-__version__ = "1.0.0"
+__version__ = "1.2.0"
 
 # We don't import Matplotlib here as this brings in NumPy. In turn, NumPy
 # caches a reference to the io.open() method as part of its data loading
@@ -257,7 +257,7 @@ class PgfutilsParser(configparser.ConfigParser):
 
     def getcolor(self, section, option, **kwargs):
         """Return a configuration entry as a Matplotlib color.
- 
+
         Recognised color formats are:
             * Named colors (red, yellow, etc.)
             * Cycle colors (C1, C2 etc.)
@@ -265,7 +265,7 @@ class PgfutilsParser(configparser.ConfigParser):
               [0, 1]
             * A floating-point value in [0, 1] for grayscale
             * 'none', 'transparent', or an empty value for transparent.
- 
+
         Parameters
         ----------
         section, option: string
@@ -348,6 +348,8 @@ def _config_reset():
            'engine': 'xelatex',
            'text_width': '345 points',
            'text_height': '550 points',
+           'marginpar_width': '65 points',
+           'marginpar_sep': '11 points',
            'num_columns': '1',
            'columnsep': '10 points',
         },
@@ -495,7 +497,8 @@ def _list_opened_files():
 _interactive = False
 
 
-def setup_figure(width=1.0, height=1.0, columns=None, **kwargs):
+def setup_figure(width=1.0, height=1.0, columns=None, margin=False,
+                 full_width=False, **kwargs):
     """Set up matplotlib figures for PGF output.
 
     This function should be imported and run before you import any matplotlib
@@ -504,16 +507,28 @@ def setup_figure(width=1.0, height=1.0, columns=None, **kwargs):
 
     Parameters
     ----------
-    width, height: float or string
+    width, height : float or string
         If a float, the fraction of the corresponding text width or height that
         the figure should take up. If a string, a dimension in centimetres
         (cm), millimetres (mm), inches (in) or points (pt). For example, '3in'
         or '2.5 cm'.
-    columns: integer, optional
+    columns : integer, optional
         The number of columns the figure should span. This should be between 1
         and the total number of columns in the document (as specified in the
         configuration). A value of None corresponds to spanning all columns.
         Any other value results in a ValueError being raised.
+    margin : Boolean, default False
+        If True, a margin figure (i.e., one to fit within the margin notes in
+        the document) is generated. If the width is a fraction, it is treated
+        as a fraction of the marginpar_width configuration setting. The height
+        is still treated as a fraction of the text height. The columns setting
+        is ignored if this is True.
+    full_width : Boolean, default False
+        If True, a full-width figure, i.e., one spanning the main text, the
+        margin notes, and the separator between them, is generated. A
+        fractional width is treated relative to the full width. The height is
+        still treated as a fraction of the text height. The columns and margin
+        parameters are ignored if this is True.
 
     """
     global _config, _interactive
@@ -620,10 +635,20 @@ def setup_figure(width=1.0, height=1.0, columns=None, **kwargs):
     # the width corresponding to the figure parameter being 1.
     # First, look up some document properties.
     text_width = _config['tex'].getdimension('text_width')
+    margin_width = _config['tex'].getdimension('marginpar_width')
+    margin_sep = _config['tex'].getdimension('marginpar_sep')
     num_columns = _config['tex'].getint('num_columns')
 
+    # Full-width figure.
+    if full_width:
+        available_width = text_width + margin_sep + margin_width
+
+    # Making a margin figure.
+    elif margin:
+        available_width = margin_width
+
     # Columns not specified, or spanning all available.
-    if columns is None or columns == num_columns:
+    elif columns is None or columns == num_columns:
         available_width = text_width
 
     # More columns than present in the document.
@@ -788,73 +813,76 @@ def save(figure=None):
     # any required modifications.
     pp_funcs = []
 
+    # Local cache of postprocessing options.
+    fix_raster_paths = _config['postprocessing'].getboolean('fix_raster_paths')
+    tikzpicture = _config['postprocessing'].getboolean('tikzpicture')
+
     # Add the appropriate directory prefix to all raster images
     # included via \pgfimage.
-    if _config['postprocessing'].getboolean('fix_raster_paths'):
+    if fix_raster_paths:
         figdir = os.path.dirname(figname) or '.'
 
         # Only apply this if the figure is not in the top-level directory.
         if not os.path.samefile(figdir, os.curdir):
             prefix = os.path.relpath(figdir)
-            expr = re.compile(r"(\\pgfimage(?:\[.+?\])?{)(.+?)}")
+            expr = re.compile(r"(\\(?:pgfimage|includegraphics)(?:\[.+?\])?{)(.+?)}")
             repl = r"\1{0:s}/\2}}".format(prefix)
             pp_funcs.append(lambda s: re.sub(expr, repl, s))
 
     # Use the tikzpicture environment rather than pgfpicture.
-    if _config['postprocessing'].getboolean('tikzpicture'):
+    if tikzpicture:
         expr = re.compile(r"\\(begin|end){pgfpicture}")
         repl = r"\\\1{tikzpicture}"
         pp_funcs.append(lambda s: re.sub(expr, repl, s))
 
     # Postprocess the figure, moving it into the final destination.
     with open(mpname, 'r') as infile, open(figname, 'w') as outfile:
-        # Update the creator line at the start of the header.
-        line = infile.readline()[:-1]
-        outfile.write(line)
-        outfile.write(" v")
-        outfile.write(mpl_version)
-        outfile.write(", matplotlib-pgfutils v")
-        outfile.write(__version__)
-        outfile.write('\n%%  Script: ')
-        outfile.write(os.path.abspath(script))
-        outfile.write('\n')
 
-        # Update the \input instructions.
-        outfile.write(infile.readline())
-        outfile.write(infile.readline())
-        _ = infile.readline()
-        outfile.write("%%   \\input{")
-        outfile.write(figname)
-        outfile.write("}\n")
-
-        # Copy the preamble instructions.
-        outfile.write(infile.readline())
-        outfile.write(infile.readline())
-        line = infile.readline()
-        if _config['postprocessing'].getboolean('tikzpicture'):
-            outfile.write("%%   \\usepackage{tikz}\n")
-        else:
-            outfile.write(line)
-        outfile.write(infile.readline())
-
-        # If we've updated the raster image directory, we can delete the
-        # instructions about getting them to appear.
-        if _config['postprocessing'].getboolean('fix_raster_paths'):
-            for n in range(7):
-                _ = infile.readline()
-
-        # Otherwise we need to copy them over.
-        else:
-            for n in range(7):
-                outfile.write(infile.readline())
-
-        # Ignore the rest of the header (which includes the preamble which was
-        # used). The first non-header line is \begingroup which we don't want
-        # to change anyway.
+        # Make some modifications to the header.
         line = infile.readline()
         while line[0] == '%':
-            outfile.write(line)
+            # Update the creator line to include pgfutils version, and add a
+            # line with the path of the script that created the figure.
+            if "Creator:" in line:
+                outfile.write(line[:-1])
+                outfile.write(" v")
+                outfile.write(mpl_version)
+                outfile.write(", matplotlib-pgfutils v")
+                outfile.write(__version__)
+                outfile.write('\n%%  Script: ')
+                outfile.write(os.path.abspath(script))
+                outfile.write('\n')
+
+            # Update the \input instructions.
+            elif r"\input{<filename>.pgf}" in line:
+                outfile.write("%%   \\input{")
+                outfile.write(figname)
+                outfile.write("}\n")
+
+            # If we're changing the figure to use the tikzpicture environment, we also
+            # need to update the required package.
+            elif tikzpicture and r"\usepackage{pgf}" in line:
+                outfile.write("%%    \\usepackage{tikz}\n")
+
+            # If we're fixing the paths to rasterised images, we can remove the
+            # instructions about using the import package.
+            elif fix_raster_paths and line.startswith("%% Figures using additional raster"):
+                while r"\import{<path to file>}" not in line:
+                    line = infile.readline()
+
+                # Discard blank line after the statement too.
+                infile.readline()
+
+            # Copy the original line.
+            else:
+                outfile.write(line)
+
+            # Next line of header.
             line = infile.readline()
+
+        # Post-process and write the first line after the header.
+        for func in pp_funcs:
+            line = func(line)
         outfile.write(line)
 
         # Apply the postprocessing to the remainder of the file.
